@@ -1,7 +1,7 @@
 #!/usr/local/bin/python2.5
 
-## Copyright (C) 2007, 2008 Tim Waugh <twaugh@redhat.com>
-## Copyright (C) 2007, 2008 Red Hat, Inc.
+## Copyright (C) 2007, 2008, 2009 Tim Waugh <twaugh@redhat.com>
+## Copyright (C) 2007, 2008, 2009 Red Hat, Inc.
 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -168,7 +168,6 @@ class JobViewer (monitor.Watcher):
         self.num_jobs_when_hidden = 0
         self.connecting_to_device = {} # dict of printer->time first seen
         self.state_reason_notifications = {}
-        self.auth_notifications = {} # by job ID
         self.auth_info_dialogs = {} # by job ID
         self.job_creation_times_timer = None
         self.special_status_icon = False
@@ -313,7 +312,6 @@ class JobViewer (monitor.Watcher):
 
         # Close any open notifications.
         for l in [self.new_printer_notifications.values (),
-                  self.auth_notifications.values (),
                   self.state_reason_notifications.values ()]:
             for notification in l:
                 if notification.get_data ('closed') != True:
@@ -566,7 +564,6 @@ class JobViewer (monitor.Watcher):
         # Check whether authentication is required.
         if self.trayicon:
             if (job_requires_auth and
-                not self.auth_notifications.has_key (job) and
                 not self.auth_info_dialogs.has_key (job)):
                 try:
                     cups.require ("1.9.37")
@@ -575,43 +572,7 @@ class JobViewer (monitor.Watcher):
                                 "authenticateJob() not available")
                     return
 
-                title = _("Authentication Required")
-                text = _("Job requires authentication to proceed.")
-                notification = pynotify.Notification (title, text, 'printer')
-                notification.set_data ('job-id', job)
-                notification.set_urgency (pynotify.URGENCY_NORMAL)
-                notification.set_timeout (pynotify.EXPIRES_NEVER)
-                notification.connect ('closed',
-                                      self.on_auth_notification_closed)
-                notification.add_action ("authenticate", _("Authenticate"),
-                                         self.on_auth_notification_authenticate)
-                self.auth_notifications[job] = notification
-                debugprint ("auth notification opened for job %s" % job)
-                self.set_statusicon_visibility ()
-
-                # In set_statusicon_visibility we process pending
-                # events, so we need to check that we still have a
-                # notification to show.
-                if notification.get_data ('closed') != True:
-                    notification.attach_to_status_icon (self.statusicon)
-                    notification.show ()
-            elif (not job_requires_auth and
-                  self.auth_notifications.has_key (job)):
-                debugprint ("job %s no longer requires auth" % job)
-                self.auth_notifications[job].close ()
-                self.auth_notifications[job].set_data ('closed', True)
-                del self.auth_notifications[job]
-
-    def on_auth_notification_closed (self, notification, reason=None):
-        job = notification.get_data ('job-id')
-        debugprint ("auth notification closed for job %s" % job)
-        self.auth_notifications[job].set_data ('closed', True)
-        del self.auth_notifications[job]
-
-    def on_auth_notification_authenticate (self, notification, action):
-        job = notification.get_data ('job-id')
-        debugprint ("auth notification authenticate for job %s" % job)
-        self.display_auth_info_dialog (job)
+                self.display_auth_info_dialog (job)
 
     def display_auth_info_dialog (self, job):
         data = self.jobs[job]
@@ -689,15 +650,11 @@ class JobViewer (monitor.Watcher):
         dialog.show_all ()
         dialog.set_keep_above (True)
         dialog.show_now ()
-        gtk.gdk.keyboard_grab (dialog.window, True)
-        gtk.gdk.pointer_grab (dialog.window, True)
 
     def auth_info_dialog_delete (self, dialog, event):
         self.auth_info_dialog_response (dialog, gtk.RESPONSE_CANCEL)
 
     def auth_info_dialog_response (self, dialog, response):
-        gtk.gdk.pointer_ungrab ()
-        gtk.gdk.keyboard_ungrab ()
         jobid = dialog.get_data ('job-id')
         del self.auth_info_dialogs[jobid]
         if response != gtk.RESPONSE_OK:
@@ -715,7 +672,6 @@ class JobViewer (monitor.Watcher):
             debugprint ("Error connecting to CUPS for authentication")
         except cups.IPPError, (e, m):
             self.show_IPP_Error (e, m)
-            pass
 
         dialog.destroy ()
 
@@ -730,11 +686,10 @@ class JobViewer (monitor.Watcher):
             return
 
         open_notifications = len (self.new_printer_notifications.keys ())
-        open_notifications += len (self.auth_notifications.keys ())
         for reason, notification in self.state_reason_notifications.iteritems():
             if notification.get_data ('closed') != True:
                 open_notifications += 1
-        num_jobs = len (self.jobs.keys ())
+        num_jobs = len (self.active_jobs)
 
         debugprint ("open notifications: %d" % open_notifications)
         debugprint ("num_jobs: %d" % num_jobs)
@@ -1078,7 +1033,11 @@ class JobViewer (monitor.Watcher):
         if not self.jobiters.has_key (jobid):
             self.add_job (jobid, jobdata)
 
-        self.active_jobs.add (jobid)
+        if self.job_is_active (jobdata):
+            self.active_jobs.add (jobid)
+        elif jobid in self.active_jobs:
+            self.active_jobs.remove (jobid)
+
         self.update_status (have_jobs=True)
         if self.trayicon:
             if not self.job_is_active (jobdata):
@@ -1098,10 +1057,13 @@ class JobViewer (monitor.Watcher):
             printer = uri
         jobdata['job-printer-name'] = printer
 
+        any_active = len (self.active_jobs) > 0
         if self.job_is_active (jobdata):
             self.active_jobs.add (jobid)
         elif jobid in self.active_jobs:
             self.active_jobs.remove (jobid)
+        if (len (self.active_jobs) > 0) != any_active:
+            self.update_status ()
 
         if (jobdata.has_key ('job-hold-until') and
             not event.has_key ('job-hold-until')):
@@ -1213,11 +1175,6 @@ class JobViewer (monitor.Watcher):
         if jobid in self.active_jobs:
             self.active_jobs.remove (jobid)
 
-        if self.auth_notifications.has_key (jobid):
-            self.auth_notifications[jobid].close ()
-            self.auth_notifications[jobid].set_data ('closed', True)
-            del self.auth_notifications[jobid]
-
         self.update_status ()
 
     def state_reason_added (self, mon, reason):
