@@ -1,7 +1,7 @@
 #!/usr/local/bin/python2.5
 
-## Copyright (C) 2007, 2008 Tim Waugh <twaugh@redhat.com>
-## Copyright (C) 2007, 2008 Red Hat, Inc.
+## Copyright (C) 2007, 2008, 2009 Tim Waugh <twaugh@redhat.com>
+## Copyright (C) 2007, 2008, 2009 Red Hat, Inc.
 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -121,7 +121,7 @@ class Monitor:
     DBUS_IFACE="com.redhat.PrinterSpooler"
 
     def __init__(self, watcher, bus=None, my_jobs=True, specific_dests=None,
-                 monitor_jobs=True):
+                 monitor_jobs=True, host=None, port=None, encryption=None):
         self.watcher = watcher
         self.my_jobs = my_jobs
         self.specific_dests = specific_dests
@@ -129,6 +129,19 @@ class Monitor:
         self.jobs = {}
         self.printer_state_reasons = {}
         self.printers = set()
+        self.process_pending_events = True
+        self.fetch_jobs_timer = None
+
+        if host:
+            cups.setServer (host)
+        if port:
+            cups.setPort (port)
+        if encryption:
+            cups.setEncryption (encryption)
+        self.user = cups.getUser ()
+        self.host = cups.getServer ()
+        self.port = cups.getPort ()
+        self.encryption = cups.getEncryption ()
 
         self.which_jobs = "not-completed"
         self.reasons_seen = {}
@@ -138,12 +151,17 @@ class Monitor:
         self.received_any_dbus_signals = False
 
         if bus == None:
-            bus = dbus.SystemBus ()
+            try:
+                bus = dbus.SystemBus ()
+            except dbus.exceptions.DBusException:
+                # System bus not running.
+                pass
 
-        bus.add_signal_receiver (self.handle_dbus_signal,
-                                 path=self.DBUS_PATH,
-                                 dbus_interface=self.DBUS_IFACE)
-        self.bus = bus
+        if bus != None:
+            bus.add_signal_receiver (self.handle_dbus_signal,
+                                     path=self.DBUS_PATH,
+                                     dbus_interface=self.DBUS_IFACE)
+            self.bus = bus
 
         self.sub_id = -1
         self.refresh ()
@@ -153,21 +171,52 @@ class Monitor:
 
     def cleanup (self):
         if self.sub_id != -1:
+            user = cups.getUser ()
             try:
-                c = cups.Connection ()
+                cups.setUser (self.user)
+                try:
+                    c = cups.Connection (host=self.host,
+                                         port=self.port,
+                                         encryption=self.encryption)
+                except TypeError:
+                    # Parameters to Connection() requires pycups >= 1.9.40.
+                    cups.setServer (self.host)
+                    cups.setPort (self.port)
+                    cups.setEncryption (self.encryption)
+                    c = cups.Connection ()
+
                 c.cancelSubscription (self.sub_id)
                 debugprint ("Canceled subscription %d" % self.sub_id)
             except:
                 pass
+            cups.setUser (user)
 
-        self.bus.remove_signal_receiver (self.handle_dbus_signal,
-                                         path=self.DBUS_PATH,
-                                         dbus_interface=self.DBUS_IFACE)
+        if self.bus != None:
+            self.bus.remove_signal_receiver (self.handle_dbus_signal,
+                                             path=self.DBUS_PATH,
+                                             dbus_interface=self.DBUS_IFACE)
+
+        timers = self.connecting_timers.values ()
+        for timer in [self.update_timer, self.fetch_jobs_timer]:
+            if timer:
+                timers.append (timer)
+        for timer in timers:
+            gobject.source_remove (timer)
 
         self.watcher.monitor_exited (self)
 
+    def set_process_pending (self, whether):
+        self.process_pending_events = whether
+
     def check_still_connecting(self, printer):
         """Timer callback to check on connecting-to-device reasons."""
+        if not self.process_pending_events:
+            # Defer the timer by setting a new one.
+            timer = gobject.timeout_add (200, self.check_still_connecting,
+                                         printer)
+            self.connecting_timers[printer] = timer
+            return False
+
         del self.connecting_timers[printer]
         debugprint ("Still-connecting timer fired for `%s'" % printer)
         (printer_jobs, my_printers) = self.sort_jobs_by_printer ()
@@ -206,8 +255,9 @@ class Monitor:
                     debugprint ("Connecting time: %d" % (time_now - t))
                     if time_now - t >= CONNECTING_TIMEOUT:
                         if have_processing_job:
-                            self.still_connecting.add (printer)
-                            self.watcher.still_connecting (self, reason)
+                            if printer not in self.still_connecting:
+                                self.still_connecting.add (printer)
+                                self.watcher.still_connecting (self, reason)
                             if self.connecting_timers.has_key (printer):
                                 gobject.source_remove (self.connecting_timers
                                                        [printer])
@@ -275,7 +325,8 @@ class Monitor:
                     else:
                         # Don't notify about this, as it must be stale.
                         debugprint ("Ignoring stale connecting-to-device")
-                        debugprint (pprint.pformat (printer_jobs))
+                        if get_debugging ():
+                            debugprint (pprint.pformat (printer_jobs))
 
         self.update_connecting_devices (printer_jobs)
         items = self.reasons_seen.keys ()
@@ -287,9 +338,27 @@ class Monitor:
                 self.watcher.state_reason_removed (self, reason)
 
     def get_notifications(self):
+        if not self.process_pending_events:
+            # Defer the timer callback.
+            gobject.source_remove (self.update_timer)
+            self.update_timer = gobject.timeout_add (200,
+                                                     self.get_notifications)
+            return False
+
         debugprint ("get_notifications")
+        user = cups.getUser ()
         try:
-            c = cups.Connection ()
+            cups.setUser (self.user)
+            try:
+                c = cups.Connection (host=self.host,
+                                     port=self.port,
+                                     encryption=self.encryption)
+            except TypeError:
+                # Parameters to Connection() requires pycups >= 1.9.40.
+                cups.setServer (self.host)
+                cups.setPort (self.port)
+                cups.setEncryption (self.encryption)
+                c = cups.Connection ()
 
             try:
                 try:
@@ -298,6 +367,7 @@ class Monitor:
                 except AttributeError:
                     notifications = c.getNotifications ([self.sub_id])
             except cups.IPPError, (e, m):
+                cups.setUser (user)
                 if e == cups.IPP_NOT_FOUND:
                     # Subscription lease has expired.
                     self.sub_id = -1
@@ -307,9 +377,11 @@ class Monitor:
                 self.watcher.cups_ipp_error (self, e, m)
                 return True
         except RuntimeError:
+            cups.setUser (user)
             self.watcher.cups_connection_error (self)
             return True
 
+        cups.setUser (user)
         deferred_calls = []
         jobs = self.jobs.copy ()
         for event in notifications['events']:
@@ -323,7 +395,8 @@ class Monitor:
             self.sub_seq = seq
             nse = event['notify-subscribed-event']
             debugprint ("%d %s %s" % (seq, nse, event['notify-text']))
-            debugprint (pprint.pformat (event))
+            if get_debugging ():
+                debugprint (pprint.pformat (event))
             if nse.startswith ('printer-'):
                 # Printer events
                 name = event['printer-name']
@@ -392,14 +465,17 @@ class Monitor:
                 deferred_calls.append ((self.watcher.job_added,
                                         (self, jobid, nse, event,
                                          jobs[jobid].copy ())))
-            elif nse == 'job-completed':
-                try:
-                    del jobs[jobid]
-                    deferred_calls.append ((self.watcher.job_removed,
-                                            (self, jobid, nse, event)))
-                except KeyError:
-                    pass
-                continue
+            elif (nse == 'job-completed' or
+                  (nse == 'job-state-changed' and
+                   event['job-state'] == cups.IPP_JOB_COMPLETED)):
+                if not (self.which_jobs in ['completed', 'all']):
+                    try:
+                        del jobs[jobid]
+                        deferred_calls.append ((self.watcher.job_removed,
+                                                (self, jobid, nse, event)))
+                    except KeyError:
+                        pass
+                    continue
 
             try:
                 job = jobs[jobid]
@@ -415,11 +491,13 @@ class Monitor:
             deferred_calls.append ((self.watcher.job_event,
                                    (self, jobid, nse, event, job.copy ())))
 
+        self.set_process_pending (False)
         self.update (jobs)
         self.jobs = jobs
 
         for (fn, args) in deferred_calls:
             fn (*args)
+        self.set_process_pending (True)
 
         # Update again when we're told to.  If we're getting CUPS
         # D-Bus signals, however, rely on those instead.
@@ -431,13 +509,28 @@ class Monitor:
 
         return False
 
-    def refresh(self):
+    def refresh(self, which_jobs=None, refresh_all=True):
         debugprint ("refresh")
 
+        if which_jobs != None:
+            self.which_jobs = which_jobs
+
+        user = cups.getUser ()
         try:
-            c = cups.Connection ()
+            cups.setUser (self.user)
+            try:
+                c = cups.Connection (host=self.host,
+                                     port=self.port,
+                                     encryption=self.encryption)
+            except TypeError:
+                # Parameters to Connection() requires pycups >= 1.9.40.
+                cups.setServer (self.host)
+                cups.setPort (self.port)
+                cups.setEncryption (self.encryption)
+                c = cups.Connection ()
         except RuntimeError:
             self.watcher.cups_connection_error (self)
+            cups.setUser (user)
             return
 
         if self.sub_id != -1:
@@ -461,7 +554,6 @@ class Monitor:
             events.extend (["job-created",
                             "job-completed",
                             "job-stopped",
-                            "job-progress",
                             "job-state-changed"])
 
         try:
@@ -469,26 +561,34 @@ class Monitor:
         except cups.IPPError, (e, m):
             self.watcher.cups_ipp_error (self, e, m)
 
+        cups.setUser (user)
+
         self.update_timer = gobject.timeout_add (MIN_REFRESH_INTERVAL * 1000,
                                                  self.get_notifications)
         debugprint ("Created subscription %d" % self.sub_id)
 
+        if self.monitor_jobs:
+            jobs = self.jobs.copy ()
+            if self.which_jobs not in ['all', 'completed']:
+                # Filter out completed jobs.
+                filtered = {}
+                for jobid, job in jobs.iteritems ():
+                    if job['job-state'] < cups.IPP_JOB_CANCELED:
+                        filtered[jobid] = job
+                jobs = filtered
+
+            self.fetch_first_job_id = 1
+            if self.fetch_jobs_timer:
+                gobject.source_remove (self.fetch_jobs_timer)
+            self.fetch_jobs_timer = gobject.timeout_add (5, self.fetch_jobs,
+                                                         refresh_all)
+        else:
+            jobs = {}
+
         try:
-            if self.monitor_jobs:
-                jobs = c.getJobs (which_jobs=self.which_jobs,
-                                  my_jobs=self.my_jobs)
-            else:
-                jobs = {}
             self.printer_state_reasons = collect_printer_state_reasons (c)
-            dests = c.getDests ()
-            printers = set()
-            for (printer, instance) in dests.keys ():
-                if printer == None:
-                    continue
-                if instance != None:
-                    continue
-                printers.add (printer)
-            self.printers = printers
+            dests = c.getPrinters ()
+            self.printers = set(dests.keys ())
         except cups.IPPError, (e, m):
             self.watcher.cups_ipp_error (self, e, m)
             return
@@ -504,13 +604,126 @@ class Monitor:
                 if printer not in self.specific_dests:
                     del jobs[jobid]
 
+        self.set_process_pending (False)
         self.watcher.current_printers_and_jobs (self, self.printers.copy (),
                                                 jobs.copy ())
         self.update (jobs)
-
         self.jobs = jobs
+        self.set_process_pending (True)
         return False
 
+    def fetch_jobs (self, refresh_all):
+        if not self.process_pending_events:
+            # Skip this call.  We'll get called again soon.
+            return True
+
+        user = cups.getUser ()
+        try:
+            cups.setUser (self.user)
+            try:
+                c = cups.Connection (host=self.host,
+                                     port=self.port,
+                                     encryption=self.encryption)
+            except TypeError:
+                # Parameters to Connection() requires pycups >= 1.9.40.
+                cups.setServer (self.host)
+                cups.setPort (self.port)
+                cups.setEncryption (self.encryption)
+                c = cups.Connection ()
+        except RuntimeError:
+            self.watcher.cups_connection_error (self)
+            self.fetch_jobs_timer = None
+            cups.setUser (user)
+            return False
+
+        limit = 1
+        try:
+            fetched = c.getJobs (which_jobs=self.which_jobs,
+                                 my_jobs=self.my_jobs,
+                                 first_job_id=self.fetch_first_job_id,
+                                 limit=limit)
+        except TypeError:
+            # limit requires pycups >= 1.9.42
+            fetched = c.getJobs (which_jobs=self.which_jobs,
+                                 my_jobs=self.my_jobs)
+            limit = len (fetched) + 1
+        except cups.IPPError, (e, m):
+            self.watcher.cups_ipp_error (self, e, m)
+            self.fetch_jobs_timer = None
+            cups.setUser (user)
+            return False
+
+        cups.setUser (user)
+        got = len (fetched)
+        debugprint ("Got %s jobs, asked for %s" % (got, limit))
+
+        deferred_calls = []
+        jobs = self.jobs.copy ()
+        jobids = fetched.keys ()
+        jobids.sort ()
+        if got > 0:
+            last_jobid = jobids[got - 1]
+        else:
+            last_jobid = self.fetch_first_job_id + limit
+        for jobid in xrange (self.fetch_first_job_id, last_jobid + 1):
+            try:
+                job = fetched[jobid]
+                if self.specific_dests != None:
+                    uri = job.get('job-printer-uri', '/')
+                    i = uri.rfind ('/')
+                    printer = uri[i + 1:]
+                    if printer not in self.specific_dests:
+                        raise KeyError
+
+                if jobs.has_key (jobid):
+                    fn = self.watcher.job_event
+                else:
+                    fn = self.watcher.job_added
+
+                jobs[jobid] = job
+                deferred_calls.append ((fn,
+                                        (self, jobid, '', {}, job.copy ())))
+            except KeyError:
+                # No job by that ID.
+                if jobs.has_key (jobid):
+                    del jobs[jobid]
+                    deferred_calls.append ((self.watcher.job_removed,
+                                            (self, jobid, '', {})))
+
+        jobids = jobs.keys ()
+        jobids.sort ()
+        if got < limit:
+            trim = False
+            for i in range (len (jobids)):
+                jobid = jobids[i]
+                if not trim and jobid > last_jobid:
+                    trim = True
+            
+                if trim:
+                    del jobs[jobid]
+                    deferred_calls.append ((self.watcher.job_removed,
+                                            (self, jobid, '', {})))
+
+        self.update (jobs)
+        self.jobs = jobs
+
+        for (fn, args) in deferred_calls:
+            fn (*args)
+
+        if got < limit:
+            # That's all.  Don't run this timer again.
+            self.fetch_jobs_timer = None
+            return False
+
+        # Remember where we got up to and run this timer again.
+        next = jobid + 1
+
+        while not refresh_all and self.jobs.has_key (next):
+            next += 1
+
+        self.fetch_first_job_id = next
+        return True
+
     def sort_jobs_by_printer (self, jobs=None):
         if jobs == None:
             jobs = self.jobs
