@@ -27,6 +27,7 @@ except ImportError:
     USE_OLD_CODE=True
 
 import errno
+from gettext import gettext as _
 import gobject
 import gtk
 import os
@@ -43,6 +44,7 @@ class AuthContext:
         self.use_user = user
         self.use_password = passwd
         self.use_workgroup = workgroup
+        self.dialog_shown = False
         self.parent = parent
 
     def perform_authentication (self):
@@ -69,6 +71,19 @@ class AuthContext:
 
         self.auth_called = False
 
+        if self.dialog_shown:
+            d = gtk.MessageDialog (self.parent,
+                                   gtk.DIALOG_MODAL |
+                                   gtk.DIALOG_DESTROY_WITH_PARENT,
+                                   gtk.MESSAGE_ERROR,
+                                   gtk.BUTTONS_CLOSE)
+            d.set_title (_("Not authorized"))
+            d.set_markup ('<span weight="bold" size="larger">' +
+                          _("Not authorized") + '</span>\n\n' +
+                          _("The password may be incorrect."))
+            d.run ()
+            d.destroy ()
+
         # After that, prompt
         d = gtk.Dialog ("Authentication", self.parent,
                         gtk.DIALOG_MODAL | gtk.DIALOG_NO_SEPARATOR,
@@ -109,23 +124,32 @@ class AuthContext:
         vbox.pack_start (table, False, False, 0)
         hbox.pack_start (vbox, False, False, 0)
         d.vbox.pack_start (hbox)
+        self.dialog_shown = True
         d.show_all ()
+        d.show_now ()
 
         if self.use_user == 'guest':
             self.use_user = pwd.getpwuid (os.getuid ())[0]
             debugprint ("pysmb: try as %s" % self.use_user)
         username_entry.set_text (self.use_user)
         domain_entry.set_text (self.use_workgroup)
+
+        d.set_keep_above (True)
+        gtk.gdk.pointer_grab (d.window, True)
+        gtk.gdk.keyboard_grab (d.window, True)
         response = d.run ()
-        d.hide ()
+        gtk.gdk.keyboard_ungrab ()
+        gtk.gdk.pointer_ungrab ()
 
         if response == gtk.RESPONSE_CANCEL:
             self.cancel = True
+            d.destroy ()
             return -1
 
         self.use_user = username_entry.get_text ()
         self.use_password = password_entry.get_text ()
         self.use_workgroup = domain_entry.get_text ()
+        d.destroy ()
         return 1
 
     def initial_authentication (self):
