@@ -1,4 +1,4 @@
-#!/usr/local/bin/python2.5
+#!/usr/local/bin/python2.6
 
 ## Printing troubleshooter
 
@@ -22,8 +22,8 @@
 import cups
 import os
 import tempfile
+import time
 from base import *
-from base import _
 class ErrorLogFetch(Question):
     def __init__ (self, troubleshooter):
         Question.__init__ (self, troubleshooter, "Error log fetch")
@@ -69,7 +69,10 @@ class ErrorLogFetch(Question):
                 c.getFile ('/admin/log/error_log', tmpfname)
                 success = True
             except cups.HTTPError:
-                os.remove (tmpfname)
+                try:
+                    os.remove (tmpfname)
+                except OSError:
+                    pass
 
             c._set_prompt_allowed (prompt)
             if success:
@@ -112,6 +115,17 @@ class ErrorLogFetch(Question):
         try:
             c.adminSetServerSettings (settings)
             success = True
+
+            # Now reconnect.
+            attempt = 1
+            while attempt <= 5:
+                try:
+                    time.sleep (1)
+                    c._connect ()
+                    break
+                except RuntimeError:
+                    # Connection failed
+                    attempt += 1
         except cups.IPPError:
             pass
 
