@@ -40,7 +40,6 @@ if __name__ == "__main__":
 
 import base
 from base import *
-from base import _
 
 class Troubleshooter:
     def __init__ (self, quitfn=None, parent=None):
@@ -99,6 +98,7 @@ class Troubleshooter:
         self.questions = []
         self.question_answers = []
         self.answers = {}
+        self.moving_backwards = False
 
         main.show_all ()
 
@@ -109,6 +109,11 @@ class Troubleshooter:
         except:
             self._report_traceback ()
 
+        # Delete the questions so that their __del__ hooks can run.
+        # Do this in reverse order of creation.
+        for i in xrange (len (self.questions)):
+            self.questions.pop ()
+
         self.main.hide ()
         if self.quitfn:
             self.quitfn (self)
@@ -144,6 +149,9 @@ class Troubleshooter:
         self.set_back_forward_buttons ()
         return page
 
+    def is_moving_backwards (self):
+        return self.moving_backwards
+
     def set_back_forward_buttons (self, *args):
         page = self.ntbk.get_current_page ()
         self.back.set_sensitive (page != 0)
@@ -161,6 +169,7 @@ class Troubleshooter:
             self.cancel.show ()
 
     def on_back_clicked (self, widget):
+        self.moving_backwards = True
         page = self.ntbk.get_current_page ()
         try:
             self.questions[page].disconnect_signals ()
@@ -190,6 +199,7 @@ class Troubleshooter:
             self._report_traceback ()
 
         self.set_back_forward_buttons ()
+        self.moving_backwards = False
 
     def on_forward_clicked (self, widget):
         page = self.ntbk.get_current_page ()
@@ -307,6 +317,7 @@ QUESTIONS = ["Welcome",
              "ErrorLogFetch",
              "PrinterStateReasons",
              "ErrorLogParse",
+             "Locale",
              "Shrug"]
 
 def run (quitfn=None, parent=None):
