@@ -1,4 +1,4 @@
-#!/usr/local/bin/python2.5
+#!/usr/local/bin/python2.6
 
 ## Copyright (C) 2006, 2007, 2008 Red Hat, Inc.
 ## Copyright (C) 2007, 2008 Tim Waugh <twaugh@redhat.com>
@@ -153,7 +153,7 @@ class UserDefaultPrompt:
 
     def on_response (self, dialog, response_id):
         if response_id != gtk.RESPONSE_OK:
-            dialog.hide ()
+            dialog.destroy ()
             return
 
         if self.systemwide.get_active ():
@@ -164,4 +164,4 @@ class UserDefaultPrompt:
             self.userdef.set (self.name)
             self.refresh_fn ()
 
-        dialog.hide ()
+        dialog.destroy ()
